----------
-- Locals
----------

local G = AceLibrary("Gratuity-2.0")
local BS = AceLibrary("Babble-Spell-2.2")

local busy = nil

local defaults = {
}

-----------
-- Addon Declaration
-----------

CrowBar = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0")
local addon = CrowBar

-----------
-- Addon Initialization
-----------

function addon:OnInitialize()
	self:RegisterDB("CrowBarDB")
	self:RegisterDefaults( "profile", defaults )
end

function addon:OnEnable()
	self:SetupButton()
	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "ScanBags")
	-- listen to locksmith scan bag event to update ourself when a box is unlocked
	self:RegisterEvent("LockSmith_ScanBags", "ScanBags")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "ScanBags")
	self:RegisterEvent("UNIT_SPELLCAST_SENT", "OpenStart")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "OpenStop")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "OpenStop")
	self:ScanBags()
end

function addon:OnDisable()
	self:HideButton()
end

------------
-- The gory innards
------------

function addon:ScanBags()
	if InCombatLockdown() then return end
	local itemid

	local boxbag, boxslot = nil,nil

	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			if not boxbag then
				G:Erase()
				G:SetBagItem( bag, slot )
				if G:Find( ITEM_OPENABLE, nil, nil, nil, true, true ) then -- search left lines for an exact match
					boxbag = bag
					boxslot = slot
				end
			end
		end
	end
	if boxbag then
		self:SetButton(boxbag, boxslot)
	else
		self:HideButton()
	end
end

function addon:OpenStart(unit, spell, rank, target )
	if unit == "player" and spell == BS["Opening"] then
		busy = true
	end
end

function addon:OpenStop(unit)
	if unit == "player" and busy then
		busy = nil
		self:ScheduleEvent("CrowBarScanBags", self.ScanBags, 1, self ) -- update after a second.
	end
end

function addon:SetButton(bag, slot)
	if InCombatLockdown() then 
		self:HideButton()
		return
	end
	-- secure attributes!
	local button = self.button
	button:SetAttribute( "type1", "macro" )
	button:SetAttribute( "macrotext1", "/use "..bag.." "..slot)
	button:Show()
end

function addon:HideButton()
	if not self.button then return end
	self.button:Hide()
end

function addon:SetupButton()
	if self.button then return end

	self.button = CreateFrame("Button", "CrowBarButton", UIParent, "SecureActionButtonTemplate")

	local button = self.button

	button:Hide()
	button.owner = self
	button:SetWidth( 32 )
	button:SetHeight( 32 )
	button:ClearAllPoints()
	button:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	button:EnableMouse(true)
	button:RegisterForDrag("LeftButton")
	button:RegisterForClicks("AnyUp")
	button:SetMovable(true)
	button:SetScript("OnDragStart", function() if not InCombatLockdown() and IsAltKeyDown() then this:StartMoving() end end)
	button:SetScript("OnDragStop", function() if not InCombatLockdown() then this:StopMovingOrSizing() this.owner:SavePosition() end end)

	button.background = button:CreateTexture( nil, "BACKGROUND")
	button.background:SetWidth( 32 )
	button.background:SetHeight( 32 )
	button.background:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0)
	button.background:SetTexture( "Interface\\Icons\\INV_Box_02" )
	button.background:SetTexCoord( 0.07, 0.93, 0.07, 0.93)

	self:RestorePosition()
end

function addon:SavePosition()
	local f = self.button
	local s = f:GetEffectiveScale()

	self.db.profile.posx = f:GetLeft() * s
	self.db.profile.posy = f:GetTop() * s
end

function addon:RestorePosition()
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	local f = self.button
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
end
